@@ -23,6 +23,11 @@
 
 #include <linux/uinput.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+
 int suinput_open(void);
 
 int suinput_enable_event(int uinput_fd, uint16_t ev_type, uint16_t ev_code);
@@ -42,4 +47,9 @@ int suinput_syn(int uinput_fd);
 
 int suinput_destroy(int uinput_fd);
 
+#ifdef __cplusplus
+}
+#endif
+
+
 #endif /* SUINPUT_H */
