@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # uinput - Python bindings for Linux uinput system
-# Copyright © 2012, 2013 Tuomas Räsänen <tuomasjjrasanen@tjjr.fi>
+# Copyright © 2012, 2013, 2014 Tuomas Räsänen <tuomasjjrasanen@tjjr.fi>
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,20 +19,22 @@
 Python bindings to Linux uinput system.
 
 Usage:
+>>> import uinput
 >>> events = (
->>>     uinput.BTN_JOYSTICK,
->>>     uinput.ABS_X + (0, 255, 0, 0),
->>>     uinput.ABS_Y + (0, 255, 0, 0),
->>>     )
->>> device = uinput.Device(events)
->>> device.emit(uinput.ABS_X, 5, syn=False)
->>> device.emit(uinput.ABS_Y, 5)
->>> device.emit_click(uinput.BTN_JOYSTICK)
+...     uinput.BTN_JOYSTICK,
+...     uinput.ABS_X + (0, 255, 0, 0),
+...     uinput.ABS_Y + (0, 255, 0, 0),
+...     )
+>>> with uinput.Device(events) as device:
+...     device.emit(uinput.ABS_X, 5, syn=False)
+...     device.emit(uinput.ABS_Y, 5)
+...     device.emit_click(uinput.BTN_JOYSTICK)
 """
 
 from __future__ import absolute_import
 
 import ctypes
+import errno
 import os
 import distutils.sysconfig as sysconfig
 
@@ -58,6 +60,18 @@ class _struct_uinput_user_dev(ctypes.Structure):
                 ("absflat", ctypes.c_int * _ABS_CNT),
                 ]
 
+def _open_error_handler(result, fn, args):
+    if result == -1:
+        code = ctypes.get_errno()
+        msg = "Failed to open the uinput device: %s" % os.strerror(code)
+        if code == errno.ENOENT:
+            raise OSError(code, "%s. Perhaps load the uinput kernel module first?" % msg)
+        else:
+            raise OSError(code, msg)
+    elif result < -1:
+        raise RuntimeError("unexpected return value: %s" % result)
+    return result
+
 def _error_handler(result, fn, args):
     if result == -1:
         code = ctypes.get_errno()
@@ -66,9 +80,12 @@ def _error_handler(result, fn, args):
         raise RuntimeError("unexpected return value: %s" % result)
     return result
 
+def fdopen():
+    return _libsuinput.suinput_open()
+
 _libsuinput_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "..", "_libsuinput" + sysconfig.get_config_var("SO")))
 _libsuinput = ctypes.CDLL(_libsuinput_path, use_errno=True)
-_libsuinput.suinput_open.errcheck = _error_handler
+_libsuinput.suinput_open.errcheck = _open_error_handler
 _libsuinput.suinput_enable_event.errcheck = _error_handler
 _libsuinput.suinput_create.errcheck = _error_handler
 _libsuinput.suinput_write_event.errcheck = _error_handler
@@ -148,7 +165,7 @@ class Device(object):
     """
 
     def __init__(self, events, name="python-uinput",
-                 bustype=0, vendor=0, product=0, version=0):
+                 bustype=0, vendor=0, product=0, version=0, fd=None):
         self.__events = events
         self.__uinput_fd = -1
         self.__name = name.encode()
@@ -158,7 +175,7 @@ class Device(object):
         user_dev.id.vendor = vendor
         user_dev.id.product = product
         user_dev.id.version = version
-        self.__uinput_fd = _libsuinput.suinput_open()
+        self.__uinput_fd = fd or fdopen()
         for ev_spec in self.__events:
             ev_type, ev_code = ev_spec[:2]
             _libsuinput.suinput_enable_event(self.__uinput_fd, ev_type, ev_code)
@@ -239,6 +256,34 @@ class Device(object):
         if syn:
             self.syn()
 
+    def destroy(self):
+        """Destroy the device. This method destroys the device file and closes the
+        underlying file descriptor. This method must be called exactly once for
+        each created device.
+
+        Alternatively, use ``with``-statement for example as follows:
+        >>> import uinput
+        >>> with uinput.Device([uinput.KEY_A]) as device:
+        ...     device.emit_click(uinput.KEY_A)
+
+        This ensures ``Device.destroy()`` is called at the end of the block.
+
+        """
+        try:
+            _libsuinput.suinput_destroy(self.__uinput_fd)
+        finally:
+            ## There's probably nothing more we can do. If the suinput_destroy()
+            ## has failed once, it almost certainly will fail again. Let's just
+            ## reset the descriptor and give up.
+            self.__uinput_fd = -1
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, *args):
+        self.destroy()
+        return False ## Not handling any exceptions.
+
     def __del__(self):
         if self.__uinput_fd >= 0:
-            _libsuinput.suinput_destroy(self.__uinput_fd)
+            self.destroy()
